//---------------------------------------------------------------------------

#ifndef RepositoryUserH
#define RepositoryUserH
//---------------------------------------------------------------------------

#include <vector.h>
#include <vcl.h>
#include <string>
#include "baseType.h"

using std::string;
using std::wstring;



class RepositoryUser{
  static void populateAllSubClassID(long baseClassID, vector<long> * result);
  static bool createLinkZZZ(long linkTypeId, long objId1 , long objId2);  
public :

  //enum propertyBaseType{ pbt_int , pbt_real , pbt_string , pbt_boolean, undefined};
  static void addEnumLiteralToEnum(long enumID , const string & enumLiteral);
  static void removeEnumLiteral(long enumID , const string & enumLiteral);
  static bool enumHasElem(const string & enumName , const string enumElemName);

  static bool isElementaryBaseType(char baseType);
  static bool equalsBaseTypeMyBaseTypeEBM(baseType b, char c);
  static bool isCardinality(char baseType);
  static char getEBMTypeCodeByName(const string & typeName);

  static bool objectExists(long objectID);
  static bool linkExists(long LinkTypeID, long objectID1,long objectID2 );
  static long getSideObjTypeId1(long LinkTypeID);

  static long getClassIdByName( const wstring & name );
  static long getClassIdByName( const string & name );
  static string getClassName( long classID );
  static string getTypeName( long typeID );

  static long createClass(const string & className);

  static long getAttributeIdByName(long classID, const WideString & name );
//  static propertyBaseType getAttributeType(long attrID);
  static baseType getAttributeType(long attrID);
  static baseType getAttributeType(const string & className,const string & attrName);

  static long getAssociationIdByName( long classID, const WideString & assocName );
  static long getAssociationInverse( long assocID );
  static long getAssociationSourceClassId( long assocID );
  static long getAssociationTargetClassId( long assocID );

  static vector<long> * getAllSubClassesID(const string & clasName);
  static vector<long> * getAllSubClassesID(long classID);



  //static long getClassIndexById( long classId); ieks L0_ObjectIDList
  static bool existsClass(const string & className);
  static bool existsComposition(const string & className, const string & roleName);  
  static bool existsEnum(const string & enumName);
  static bool existsAttribute(const string & clName , const string & attrName);
  static bool existsAttribute(const string & clName , const string & attrName ,
                              const string & attrTypeName );  
  static bool classExists( long classID );
  static bool assocExists( long assocID );
  static bool assocStartsFrom( long assocID , long classID );

  static long getObjectNum(long objTypeId);
  static long getObjectIdByIndex(long objTypeId , int index);

  static long getLinkedObjectNum(long objTypeId , long linkTypeId);
  static long getLinkedObjectIdByIndex(long objId , long linkTypeId, int index );
  static void getAllLinkTypes(vector<long> * result);
  static void getAllClasses(vector<long> & result);
  static vector<long> * getAllClassesIDs();

  static bool deleteObject(long objectID);
  //veido jauno objektu ar tipu classID
  //un ieliek jaunizveidota objekta ID  L0_ObjectIDList sarakstaa
  static long createObject_zzz(long classID);
  //veido jaunu objektu repozitorija un neko vairak nedara
  static long createObjectRaw(long classID);
  //veido jaunu saaiti repozitorija un neko vairak nedara
  static long createLinkRaw(long linkTypeId, long objId1 , long objId2);
  static long createLinkAfterRaw(long linkTypeId, long objId1 , long objId2, long afterPartnerID);  


  static void createLink(long linkTypeId, long objId1 , long objId2);
  static void createLinkAfter(long linkTypeID, long objID1,
                                     long objID2, long afterPartnerID);

  static bool deleteLink(long linkTypeId, long objId1 , long objId2);

  //createLink_2, ja nevar izveidot jaunu linku vienkarsi atgriez false
  //createLink , ja nevar izveidot linku rada zinojumu un assert
  // tas paats ir ar deleteLink
  static bool createLink_2(long linkTypeId, long objId1 , long objId2);
  static bool deleteLink_2(long linkTypeId, long objId1 , long objId2);






  static bool setAttributeValue(long objectID,long attributeID,const string & value);
  static bool setAttributeValue(long objectID,long attributeID,const WideString & value);

  static string getAttributeValue(long objectID,long attributeID);
  static WideString  getAttributeValue_2(long objectID,long attributeID);

  static bool hasSubClasses(const string & className);
  static bool hasSubClasses(long classID);

  static long getObjectTypeId( long objId );
  //static long getObjectNum(long objTypeId);


  static bool isParent(const string &  parentName , const string & childName );
  static bool isSuperClass(long  superClassID , long subClassID );

  static long getPackageIdByName(const string & pckName);
  static long createPackage(const string & pckName);
  static int setPackage( long pckId , long lastPckId );

  //MM changing functions
  //void createClass(const string & className);
  static void deleteClass(long classID);
  static long createAttr( const string & attrName , char baseType); //baseType from ect_data_api.h
  static long createEnum( const string & attrName );
  //void createEnumerationAttr(const string & attrName , const vector<const string> & enumElems);
  static bool deleteAttr( long attrID );

  static bool addAttr( long attrID , long classID);
  static bool removeAttr( long attrID , long classID);

  static long createAssociation(const string & subjectRoleName,
                                        const string & objectRoleName,
                                        long subjectClassID,char subjectEndCardinality,
                                        long objectClassID, char objectEndCardinality,
                                        bool fromIsOrdered, bool toIsOrdered );
                                        
  static long createComposition( const string & subjectRoleName,
                                        const string & objectRoleName,
                                        long subjectClassID,char subjectEndCardinality,
                                        long objectClassID, char objectEndCardinality,
                                        bool fromIsOrdered, bool toIsOrdered);

  static void deleteAssociation(long assocID);
  static int createGeneralization(long baseClassID , long derivedClassID);
  static int deleteGeneralization(long baseClassID , long  derivedClassID);




  static TList * getAllClassIDs();
  static TList * getAllAssocIDs(const string & className);
  static TList * getDirectAssocIDs(const string & className);
  static TList * getAllAttributeIDs(const string & className);
  static TList * getAllEnumIDs();
  static TList * getAllEnumElems(const string & enumName);    
  static TList * getDirectAttributeIDs(const string & className);
  static string  getAttrTypeName(long attrID);
  static TList * getSubClassesIDs( const string & name);
  //static TList * getAllClassIDs();
};

#endif

